
version 10
capture log close
log using dsge_forecast_eval_oct192010, t replace

************************************************************
*                                                          *
* Program to compare the forecast performance of           *
* statistical, Smets-Wouters estimated model based and     * 
* judgmential forecasts.                                   *
*                                                          *
* Edge and Gurkaynak                                       *
*                                                          *
* This version: October 19, 2010                           *
* rsg                                                      *
*                                                          *
************************************************************

******************************
*Preliminaries               *
******************************

*Include time stamp in the results log file
display "$S_DATE $S_TIME"

*Memory management
set mem 500m
set matsize 5000

*Dont stop when the screen fills while running the code
set more off

*I'll let the output file fill with output from loops replacing variables 
*because I check them to make sure that the pattern is right. 
*Uncomment the line below and the associated closing bracket, which is 
*also identified with a comment, to have tidy log file.  

 
*qui { //Keep the output of the data organization out of the sreen and the log file. 

*****************************************
*Process data and put everything into a *
*single usable .dta file.               *
*****************************************

******************************************
*The data are in .csv files, a different *
*file for each of GDP growth, inflation  *
*and interest rates.                     *
******************************************

*First convert each data file into .dta format and save so that they can be merged *
*Also rename the variables so that the names are not too long. swinff1q will be 1q ahead 
*SW model forecast and gbgdpgf4q will be the 4q ahead GDP growth forecast of the 
*Green Book. 

foreach xx in inf gdpg intr {
	insheet using BC_`xx'.csv

	capture drop _merge

	foreach yy of varlist sw`xx'fcast1q-bc`xx'fcast8q {
		local newn=subinstr("`yy'", "cast", "", 1)
		ren `yy' `newn'
	}

	sort date
	save BC_`xx'.dta, replace
	clear
}

*Now merge the three data sets*

use BC_inf.dta
merge date using BC_gdpg.dta
sort date
drop _merge
merge date using BC_intr.dta
sort date
drop _merge
  
save eg_bc_fcast_eval_data_oct52010, replace

/*

*Get the historical data for realizations of inf, gdpg and intr from BC_jan92niparff.csv
*Double the observations so that they follow the two observations per quarter convetion 
*that the forecast data set has. I don't know whether we'll use this but it won't do any harm. 
*The data set has levels of GDP and the Deflator, and annualized interest rates. 
*We'll get percentage q/q growth rates of the first two, and convert the third to quarterly rates
*to be compatible with the forecast data file.  

clear
insheet using BC_jan92niparff.csv

gen inf=(ln(gdpdef)-ln(gdpdef[_n-1]))*100
gen gdpg=(ln(gdpc)-ln(gdpc[_n-1]))*100
gen intr=((((rff/100)+1)^0.25)-1)*100

keep date intr inf gdpg
order date inf gdpg intr

count
local tot = r(N)
local newtot = 3*r(N)

set obs `newtot'

forvalues xx=1/`tot' {

	local ono=`xx'+`tot' 
	local ono2=`xx'+2*`tot'
	replace date=date[`xx']+"B" in `ono'
	replace date=date[`xx']+"C" in `ono2'
	replace inf=inf[`xx'] in `ono'
	replace gdpg=gdpg[`xx'] in `ono'
	replace intr=intr[`xx'] in `ono'
	replace inf=inf[`xx'] in `ono2'
	replace gdpg=gdpg[`xx'] in `ono2'
	replace intr=intr[`xx'] in `ono2'


}

replace date=date+"A" in 1/`tot'
sort date
save histdata.dta, replace

append using eg_bc_fcast_eval_data_oct52010

sort date
save eg_bc_fcast_eval_data_oct52010, replace


*We'll work with this combined data set from now on*

*/

******************************************
* Create date and intraquarter variables *
******************************************

/*There are two FOMC meetings per quarter. We denote the forecasts made in the first one with A and those made
  in the second one with B. Thus we have two rows of data per quarter such as 2003Q1A, 2003Q1B. These have the same 
  realized value for that quarter but the one quarter ahead forecast in A was made at the time of the first FOMC 
  meeting in the quarter and that in B was made at the time of the second FOMC meeting. 

  The convention is similar for n quarter ahead forecasts. The forecast recorded in 2003Q1A was made at the time of 
  the first meeting n quarters prior, and that in 2003Q1B was made at the time of the second meeting in the same 
  n-period prior quarter. */

*Generate variable ab to distinguish between A, B and C date forecasts. 

gen ab=substr(date, -1, 1)
label var ab "Forecasts coming from the first (A), second (B) or third (C) month in each quarter"

*Generate date variables to be used later*

gen year=substr(date, 1, 4)
gen qtr=substr(date, 6, 1)

gen yrq=substr(year, 3, 2)+"Q"+qtr

destring year, replace
destring qtr, replace

*We now have numeric year and quarter variables and string variables date and yrq which is in the 03Q1 format. 

*We will want to use some time series operators. Define id as the observation number and use that as the time dimension*

gen id=_n
tsset id

gen numdate=yq(year, qtr)
format numdate %tq

*********************************************
*Generate realized values and forecasts of  *
*CHANGES in interest rates.                 *
*********************************************

count
local tot = r(N)

gen dint=.
*label var dint "Change (q/q) in the quarterly interest rate"

forvalues xx=1/8 {

	gen swdintf`xx'q=.
	gen bvardintf`xx'q=.
	gen tbvardintf`xx'q=.
	gen bcdintf`xx'q=.
}

forvalues  xx=4/`tot' {

	local prev=`xx'-3
	replace dint=intr[`xx']-intr[`prev'] in `xx'

	replace swdintf1q=swintrf1q[`xx']-intr[`prev'] in `xx'
	replace bvardintf1q=bvarintrf1q[`xx']-intr[`prev'] in `xx'
	replace tbvardintf1q=tbvarintrf1q[`xx']-intr[`prev'] in `xx'
	replace bcdintf1q=bcintrf1q[`xx']-intr[`prev'] in `xx'

	forvalues yy=2/8 {

		local lag=`yy'-1
		replace swdintf`yy'q=swintrf`yy'q[`xx']-swintrf`lag'q[`prev'] in `xx'
		replace bvardintf`yy'q=bvarintrf`yy'q[`xx']-bvarintrf`lag'q[`prev'] in `xx'
		replace tbvardintf`yy'q=tbvarintrf`yy'q[`xx']-tbvarintrf`lag'q[`prev'] in `xx'
		replace bcdintf`yy'q=bcintrf`yy'q[`xx']-bcintrf`lag'q[`prev'] in `xx'

	}
}
	



*} //THIS IS THE CLOSING BRACKET FOR THE QUI. Uncomment this if you uncommented the qui way at the beginning.  	




**************************
*Generate forecast errors*
**************************

*Produce the CONSTANT forecast
*The constant forecast uses the realized mean of the relavant variable in 1992-2010

foreach xx of varlist inf gdpg intr {
	sum `xx' if bc`xx'f4q~=.
	gen cf`xx'f=r(mean) if year>1991
	disp "Constant forecast (forecast sample mean) of `xx' is: " r(mean)
}

gen cfdintf=0 if year>1991

disp "Constant forecast (forecast sample mean) of dintr is: 0" 


*Produce the RANDOM WALK forecast

*The observation number corresponding to first forecast `first'.

levelsof id if date=="1992Q1A"
local first = r(levels)



foreach xx of varlist inf gdpg intr {
	forvalues yy=1/8	{	
		
		gen rw`xx'f`yy'q=.

		forvalues zz=`first'/`tot' {
			
			local prev=`zz'-(3*`yy')
			replace rw`xx'f`yy'q=`xx'[`prev'] in `zz'
			}	
	}	
}



forvalues yy=1/8	{

	gen rwdintf`yy'q=0 in `first'/l	
}


*eXYZ variables will hold the forecast errors of the relevant variables. 
*For example errswgdpgfacst4q has the 4-qtr ahead SW model forecast error for GDP growth 

foreach xx in inf gdpg intr dint {

	foreach yy of varlist sw`xx'f1q-bc`xx'f8q rw`xx'f1q-rw`xx'f8q{
		gen e`yy'=`xx'-`yy'
		label var e`yy' "Forecast error of `yy'"
		gen sqe`yy'=e`yy'^2
		label var sqe`yy' "Squared forecast error of `yy'"
	}
	gen ecf`xx'f=`xx'-cf`xx'f
	gen sqecf`xx'f=ecf`xx'f^2

}



*Produce RMSE and Relative RMSE estimates
*

gen quarters=0

foreach yy in sw bvar tbvar bc cf rw {

	foreach zz in inf gdpg intr dint {
		gen `yy'`zz'rmse=0
	}
}

forvalues xx=1/8 {

	replace quarters=`xx' in `xx' 

	foreach zz in inf gdpg intr dint {

		foreach yy in sw bvar tbvar bc rw {

			sum sqe`yy'`zz'f`xx'q if sqebc`zz'f`xx'q~=.
			replace `yy'`zz'rmse=sqrt(r(mean)) in `xx'	
		}
	
	sum sqecf`zz'f if sqebc`zz'f`xx'q~=.
	replace cf`zz'rmse=sqrt(r(mean)) in `xx'
	}			
			
}

*Produce relative RMSE

foreach zz in inf gdpg intr dint {
	
	gen swbcrel`zz'rmse=sw`zz'rmse/bc`zz'rmse in 1/8
	gen swbvarrel`zz'rmse=sw`zz'rmse/bvar`zz'rmse in 1/8
}


/*

**************
*Make FIGURES*
**************


*Produce Relative RMSE graphs--Figure 1

graph twoway scatter swbcrelinfrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. Inflation Forecast RMSE) name(sbcinfrmse)
graph twoway scatter swbcrelgdpgrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. GDP Growth Forecast RMSE) name(sbcgdpgrmse)
graph twoway scatter swbvarrelinfrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. Inflation Forecast RMSE) name(sbvinfrmse)
graph twoway scatter swbvarrelgdpgrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. GDP Growth Forecast RMSE) name(sbvgdpgrmse)

graph combine sbcinfrmse sbcgdpgrmse, cols(1) title("DSGE Model Relative to BC") name(sbc) ycommon 
graph combine sbvinfrmse sbvgdpgrmse, cols(1) title("DSGE Model Relative to BVAR") name(sbv) ycommon
graph combine sbc sbv, cols(2) title("Figure 1. Relative Forecast RMSEs") name(Figure1) saving("Figure1.gph", replace)
graph export Figure1.wmf, replace


*Produce scatter graphs--Figure 2

scatter inf swinff4q if bcinff4q~=., xsc(r(0 1)) ysc(r(0 1)) xlabel(0(0.2)1) ylabel(0(0.2)1) xtitle("DSGE Model Forecast of Inflation") ytitle(Realized Inflation) name(sw4qinf)
scatter gdpg swgdpgf4q if bcinff4q~=., xlabel(-01(0.5)2) ylabel(-1(0.5)2) xtitle("DSGE Model Forecast of GDP Growth") ytitle(Realized GDP Growth) name(sw4qgdp)

graph combine sw4qinf sw4qgdp, cols(1) title("Figure 2. Realized and 4Q Ahead DSGE Model Forecasts") name(Figure2) saving("Figure2.gph", replace)
graph export Figure2.wmf, replace


*Produce RMSE graphs--Figure 3

graph twoway scatter swinfrmse bvarinfrmse bcinfrmse cfinfrmse quarters in 1/8, msymbol(X sh Dh Oh) mcolor(black black black black) ylabel(0.1(0.1)0.4) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Inflation) legend(off) name(infrmse)
graph twoway scatter swgdpgrmse bvargdpgrmse bcgdpgrmse cfgdpgrmse quarters in 1/8, msymbol(X sh Dh Oh) mcolor(black black black black) ylabel(0.4(0.1)0.7) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(GDP Growth) legend(off) name(gdpgrmse)
graph twoway scatter swdintrmse bvardintrmse bcdintrmse cfdintrmse quarters in 1/8, ylabel(0(0.05)0.15) legend(label(1 "DSGE") label(2 "BVAR") label(3 "Blue Chip") label(4 "Constant") rows(1) ring(0) position(5)) msymbol(X sh Dh Oh) mcolor(black black black black) xlabel(1(1)8) xtitle("Quarters") ytitle(Interest Rate Change) name(dintrmse)

graph combine infrmse gdpgrmse dintrmse, cols(1) title("Figure 3. RMSEs of Alternative Forecasts") name(Figure3) saving("Figure3.gph", replace)
graph export Figure3.wmf, replace

*Produce inflation graph--Figure 4

graph twoway line inf numdate if ab=="A", ytitle("Inflation") xtitle("Date") title("Figure 4. A Short History of Inflation") xlabel( #14, angle(vertical)) xline(20 131, lwidth(0.5)) xline(148 187, lwidth(1)) name(tsinf) saving(Figure4.gph, replace)
graph export Figure4.wmf, replace


*************
*Make TABLES*
*************


foreach yy in sw bvar bc {

	foreach zz in inf gdpg intr dint {
	
	newey `zz', lag(3)
	outreg using `yy'_`zz'_newey.out, replace
	reg `zz'
	outreg using `yy'_`zz'_reg.out, replace
	
	}
}


forvalues xx=1/6 {

	foreach yy in sw bvar bc {

		foreach zz in inf gdpg intr dint {
		
		label var `yy'`zz'f`xx'q "`yy'`zz'fcast"
		local lag=`xx'*3
		newey `zz' `yy'`zz'f`xx'q, lag(`lag') force
		outreg using `yy'_`zz'_newey.out, coefastr se append
		reg `zz' `yy'`zz'f`xx'q, cluster(yrq)
		outreg using `yy'_`zz'_reg.out, coefastr se append

		}
	}
}


gen swcfinf=.
gen swcfgdpg=.
gen swcfdint=.

forvalues xx=1/8 {

	foreach yy in inf gdpg dint {
	
	dmariano `yy' sw`yy'f`xx'q cf`yy'f if bc`yy'f`xx'q~=., kernel(bartlett)
	replace swcf`yy'=r(p) in `xx'
	}
}


*/

*Nonoverlapping forecast errors

graph twoway scatter  eswinff4q  year if  year>1992 & qtr==1 & ab=="A", msymbol(X) mcolor(black) yline(0.25, lc(black)) yline(-0.25, lc(black)) ylabel(-0.5(0.25)0.5) xlabel(1993(1)2010, angle(90)) xtitle("Years") ytitle(DSGE inflation Forecast Error) legend(off) c(l) lc(black) lp(".") name(swinferr)
graph twoway scatter  eswgdpgf4q  year if  year>1992 & qtr==1 & ab=="A", msymbol(X) mcolor(black) yline(0.25, lc(black)) yline(-0.25, lc(black)) ylabel(-2(1)2) xlabel(1993(1)2010, angle(90)) xtitle("Years") ytitle(DSGE GDP Growth Forecast Error) legend(off) c(l) lc(black) lp(".") name(swgdperr)
graph combine swinferr swgdperr, cols(1) title("Figure 5. Non-Overlapping 4Q Ahead Forecast Errors") name(Figure5) saving("Figure5.gph", replace)
graph export Figure5.wmf, replace




*Rolling averages of RMSE
gen rollyr=.
gen swrolinf=.
gen swrolgdp=.
gen bcrolinf=.
gen bcrolgdp=.
gen bvarrolinf=.
gen bvarrolgdp=.



forvalues xx=1993/2009 {
	
	local dum=`xx'-1992
	replace rollyr=`xx' in `dum'

	foreach yy in sw bc bvar { 
		
		sum sqe`yy'inff4q if year>(`xx'-2) & year<(`xx'+2)
		replace `yy'rolinf=sqrt(r(mean)) in `dum'
		sum sqe`yy'gdpgf4q if year>(`xx'-2) & year<(`xx'+2)
		replace `yy'rolgdp=sqrt(r(mean)) in `dum'
	}

}

graph twoway scatter  swrolinf bvarrolinf bcrolinf rollyr if  rollyr~=., msymbol(X sh Dh Oh) mcolor(black black black) ylabel(0(0.1)0.4) xlabel(1993(1)2009, angle(90)) xtitle("Years") ytitle(Rolling Inflation RMSE) legend(off) name(rolinf)
graph twoway scatter  swrolgdp bvarrolgdp bcrolgdp rollyr if  rollyr~=., msymbol(X sh Dh Oh) mcolor(black black black) ylabel(0(0.25)1.25) xlabel(1993(1)2009, angle(90)) xtitle("Years") ytitle(Rolling GDP Growth RMSE) legend(label(1 "DSGE") label(2 "BVAR") label(3 "Blue Chip") rows(1) ring(0) position(5)) name(rolgdp)
graph combine rolinf rolgdp, cols(1) title("Figure 6. Three Year Rolling Average of 4Q Ahead RMSEs") name(Figure6) saving("Figure6.gph", replace)
graph export Figure6.wmf, replace




*Figures of forecasts and the realized over time

*Choose the beginning quarter
levelsof id if date=="2007Q4C"
local firstq=r(levels)
*+24 makes the code create variables for the initial and 8 subsequent quarters 
local lastq=`firstq'+24

forvalues xx=`firstq'(3)`lastq' {
	local thisq=date[`xx']
	gen Z`thisq'inf=inf[`xx'] in 1/12
	gen Z`thisq'gdpg=gdpg[`xx'] in 1/12
	gen wf`thisq'=""
	gen sw`thisq'inff=.
	gen bvar`thisq'inff=.
	gen bc`thisq'inff=.
	gen sw`thisq'gdpgf=.
	gen bvar`thisq'gdpgf=.
	gen bc`thisq'gdpgf=.
	

	forvalues yy=11(-1)0 {
		
		local obsno=12-`yy'
		local index=`xx'-`yy'
		replace wf`thisq'=date[`index'] in `obsno'
		
	}

	
	local afor=`xx'-2
	local bfor=`xx'-1

	
	foreach yy in sw bvar bc {
 
		foreach zz in inf gdpg {

			replace `yy'`thisq'`zz'f=`yy'`zz'f4q[`afor'] in 1
			replace `yy'`thisq'`zz'f=`yy'`zz'f4q[`bfor'] in 2
			replace `yy'`thisq'`zz'f=`yy'`zz'f4q[`xx'] in 3

			replace `yy'`thisq'`zz'f=`yy'`zz'f3q[`afor'] in 4
			replace `yy'`thisq'`zz'f=`yy'`zz'f3q[`bfor'] in 5
			replace `yy'`thisq'`zz'f=`yy'`zz'f3q[`xx'] in 6

			replace `yy'`thisq'`zz'f=`yy'`zz'f2q[`afor'] in 7
			replace `yy'`thisq'`zz'f=`yy'`zz'f2q[`bfor'] in 8
			replace `yy'`thisq'`zz'f=`yy'`zz'f2q[`xx'] in 9

			replace `yy'`thisq'`zz'f=`yy'`zz'f1q[`afor'] in 10
			replace `yy'`thisq'`zz'f=`yy'`zz'f1q[`bfor'] in 11
			replace `yy'`thisq'`zz'f=`yy'`zz'f1q[`xx'] in 12

		}
	}
}

	


foreach xx in 2007Q4C 2008Q1C 2008Q2C 2008Q3C 2008Q4C 2009Q1C 2009Q2C {

	*First create label values that will appear on the horizontal axis
	capture label drop zz
	local dene=wf`xx'[1]
	label def zz 1 "`dene'"

	forvalues i=2/12 {
		local dene= wf`xx'[`i']
		label def zz `i' "`dene'", add
	}

	
	*Attach labels to horizontal axis values 
	label val id zz
	
	local titl=substr("`xx'",1,6)

	scatter  sw`xx'inff bvar`xx'inff bc`xx'inff  Z`xx'inf id in 1/12, t1title("`titl' Inflation") xtitle("Forecast date") msymbol(X sh Dh i) mcolor(black black black)  xlabel(1(1)12, valuelabel) xlabel(1(1)12, angle(90)) c(l l l l) lc(black black black black) lp("." "--" "-." l)legend(label(1 "DSGE") label(2 "BVAR") label(3 "Blue Chip")  label(4 "Realized") rows(2) ring(0) position(8)) name(F`xx'inf)
	scatter  sw`xx'gdpgf bvar`xx'gdpgf bc`xx'gdpgf  Z`xx'gdpg id in 1/12, t1title("`titl' GDP Growth") xtitle("Forecast date") msymbol(X sh Dh i) mcolor(black black black)  xlabel(1(1)12, valuelabel) xlabel(1(1)12, angle(90)) c(l l l l) lc(black black black black) lp("." "--" "-." l)legend(label(1 "DSGE") label(2 "BVAR") label(3 "Blue Chip")  label(4 "Realized") rows(2) ring(0) position(8)) name(F`xx'gdp)


}

	graph combine F2007Q4Cinf F2008Q3Cinf F2009Q1Cinf, title("Figure 7a. Forecasts of Inflation during the Crisis") name(Figure7a) saving("Figure7a.gph", replace)
	graph export Figure7a.wmf, replace
	graph combine F2007Q4Cgdp F2008Q3Cgdp F2009Q1Cgdp, title("Figure 7b. Forecasts of GDP Growth during the Crisis") name(Figure7b) saving("Figure7b.gph", replace)
	graph export Figure7a.wmf, replace


log close
