
version 10
capture log close
log using dsge_forecast_eval_oct142010, t replace

************************************************************
*                                                          *
* Program to compare the forecast performance of           *
* statistical, Smets-Wouters estimated model based and     * 
* judgmential forecasts.                                   *
*                                                          *
* Edge and Gurkaynak                                       *
*                                                          *
* This version: October 14, 2010                            *
* rsg                                                      *
*                                                          *
************************************************************

******************************
*Preliminaries               *
******************************

*Include time stamp in the results log file
display "$S_DATE $S_TIME"

*Memory management
set mem 500m
set matsize 5000

*Dont stop when the screen fills while running the code
set more off

*I'll let the output file fill with output from loops replacing variables 
*because I check them to make sure that the pattern is right. 
*Uncomment the line below and the associated closing bracket, which is 
*also identified with a comment, to have tidy log file.  

 
*qui { //Keep the output of the data organization out of the sreen and the log file. 

*****************************************
*Process data and put everything into a *
*single usable .dta file.               *
*****************************************

******************************************
*The data are in .csv files, a different *
*file for each of GDP growth, inflation  *
*and interest rates.                     *
******************************************

*First convert each data file into .dta format and save so that they can be merged *
*Also rename the variables so that the names are not too long. swinff1q will be 1q ahead 
*SW model forecast and gbgdpgf4q will be the 4q ahead GDP growth forecast of the 
*Green Book. 

foreach xx in inf gdpg intr {
	insheet using GB_`xx'.csv

	capture drop _merge

	foreach yy of varlist sw`xx'fcast1q-gb`xx'fcast8q {
		local newn=subinstr("`yy'", "cast", "", 1)
		ren `yy' `newn'
	}

	sort date
	save GB_`xx'.dta, replace
	clear
}

*Now merge the three data sets*

use GB_inf.dta
merge date using GB_gdpg.dta
sort date
drop _merge
merge date using GB_intr.dta
sort date
drop _merge
  
save eg_fcast_eval_data_oct82010, replace

*Get the historical data for realizations of inf, gdpg and intr from swjan97niparff.csv
*Double the observations so that they follow the two observations per quarter convetion 
*that the forecast data set has. I don't know whether we'll use this but it won't do any harm. 
*The data set has levels of GDP and the Deflator, and annualized interest rates. 
*We'll get percentage q/q growth rates of the first two, and convert the third to quarterly rates
*to be compatible with the forecast data file.  

clear
insheet using swjan92niparff.csv

gen inf=(ln(gdpdef)-ln(gdpdef[_n-1]))*100
gen gdpg=(ln(gdpc)-ln(gdpc[_n-1]))*100
gen intr=((((rff/100)+1)^0.25)-1)*100

keep date intr inf gdpg
order date inf gdpg intr

count
local tot = r(N)
local newtot = 2*r(N)

set obs `newtot'

forvalues xx=1/`tot' {

	local ono=`xx'+`tot' 
	replace date=date[`xx']+"B" in `ono'
	replace inf=inf[`xx'] in `ono'
	replace gdpg=gdpg[`xx'] in `ono'
	replace intr=intr[`xx'] in `ono'

}

replace date=date+"A" in 1/`tot'
sort date
save histdata.dta, replace

append using eg_fcast_eval_data_oct82010

sort date
save eg_fcast_eval_data_oct82010, replace


*We'll work with this combined data set from now on*



******************************************
* Create date and intraquarter variables *
******************************************

/*There are two FOMC meetings per quarter. We denote the forecasts made in the first one with A and those made
  in the second one with B. Thus we have two rows of data per quarter such as 2003Q1A, 2003Q1B. These have the same 
  realized value for that quarter but the one quarter ahead forecast in A was made at the time of the first FOMC 
  meeting in the quarter and that in B was made at the time of the second FOMC meeting. 

  The convention is similar for n quarter ahead forecasts. The forecast recorded in 2003Q1A was made at the time of 
  the first meeting n quarters prior, and that in 2003Q1B was made at the time of the second meeting in the same 
  n-period prior quarter. */

*Generate variable ab to distinguish between A and B date forecasts. 

gen ab=substr(date, -1, 1)
label var ab "Forecasts coming from the first (A) or second (B) meeting in each quarter"

*Generate date variables to be used later*

gen year=substr(date, 1, 4)
gen qtr=substr(date, 6, 1)

gen yrq=substr(year, 3, 2)+"Q"+qtr

destring year, replace
destring qtr, replace

*We now have numeric year and quarter variables and string variables date and yrq which is in the 03Q1 format. 

*We will want to use some time series operators. Define id as the observation number and use that as the time dimension*

gen id=_n
tsset id

gen numdate=yq(year, qtr)
format numdate %tq

*********************************************
*Generate realized values and forecasts of  *
*CHANGES in interest rates.                 *
*********************************************

count
local tot = r(N)

gen dint=.
*label var dint "Change (q/q) in the quarterly interest rate"

forvalues xx=1/8 {

	gen swdintf`xx'q=.
	gen bvardintf`xx'q=.
	gen tbvardintf`xx'q=.
	gen gbdintf`xx'q=.
}

forvalues  xx=3/`tot' {

	local prev=`xx'-2
	replace dint=intr[`xx']-intr[`prev'] in `xx'

	replace swdintf1q=swintrf1q[`xx']-intr[`prev'] in `xx'
	replace bvardintf1q=bvarintrf1q[`xx']-intr[`prev'] in `xx'
	replace tbvardintf1q=tbvarintrf1q[`xx']-intr[`prev'] in `xx'
	replace gbdintf1q=gbintrf1q[`xx']-intr[`prev'] in `xx'

	forvalues yy=2/8 {

		local lag=`yy'-1
		replace swdintf`yy'q=swintrf`yy'q[`xx']-swintrf`lag'q[`prev'] in `xx'
		replace bvardintf`yy'q=bvarintrf`yy'q[`xx']-bvarintrf`lag'q[`prev'] in `xx'
		replace tbvardintf`yy'q=tbvarintrf`yy'q[`xx']-tbvarintrf`lag'q[`prev'] in `xx'
		replace gbdintf`yy'q=gbintrf`yy'q[`xx']-gbintrf`lag'q[`prev'] in `xx'

	}
}
	



*} //THIS IS THE CLOSING BRACKET FOR THE QUI. Uncomment this if you uncommented the qui way at the beginning.  	




**************************
*Generate forecast errors*
**************************

*Produce the CONSTANT forecast
*The constant forecast uses the realized mean of the relavant variable in 1992-2010

foreach xx of varlist inf gdpg intr {
	sum `xx' if gb`xx'f4q~=.
	gen cf`xx'f=r(mean) if year>1991
	disp "Constant forecast (forecast sample mean) of `xx' is: " r(mean)
}

gen cfdintf=0 if year>1991

disp "Constant forecast (forecast sample mean) of dintr is: 0" 


*Produce the RANDOM WALK forecast

*The observation number corresponding to first forecast `first'.

levelsof id if date=="1992Q1A"
local first = r(levels)



foreach xx of varlist inf gdpg intr {
	forvalues yy=1/8	{	
		
		gen rw`xx'f`yy'q=.

		forvalues zz=`first'/`tot' {
			
			local prev=`zz'-(3*`yy')
			replace rw`xx'f`yy'q=`xx'[`prev'] in `zz'
			}	
	}	
}



forvalues yy=1/8	{

	gen rwdintf`yy'q=0 in `first'/l	
}



*eXYZ variables will hold the forecast errors of the relevant variables. 
*For example errswgdpgfacst4q has the 4-qtr ahead SW model forecast error for GDP growth 

foreach xx in inf gdpg intr dint {

	foreach yy of varlist sw`xx'f1q-gb`xx'f8q rw`xx'f1q-rw`xx'f8q{
		gen e`yy'=`xx'-`yy'
		label var e`yy' "Forecast error of `yy'"
		gen sqe`yy'=e`yy'^2
		label var sqe`yy' "Squared forecast error of `yy'"
	}
	gen ecf`xx'f=`xx'-cf`xx'f
	gen sqecf`xx'f=ecf`xx'f^2

}

*Produce RMSE and Relative RMSE estimates
*The RMSE and related quantities are stored as observations
*1 to 8 in new variables, and the remainder of the observations for
*these variables are missing. 
*This is ugly but it circumvents a lot of problems. 

gen quarters=0

foreach yy in sw bvar tbvar gb cf rw {

	foreach zz in inf gdpg intr dint {
		gen `yy'`zz'rmse=0
	}
}

forvalues xx=1/8 {

	replace quarters=`xx' in `xx' 

	foreach zz in inf gdpg intr dint {

		foreach yy in sw bvar tbvar gb rw{

			sum sqe`yy'`zz'f`xx'q if sqegb`zz'f`xx'q~=.
			replace `yy'`zz'rmse=sqrt(r(mean)) in `xx'	
		}
	
	sum sqecf`zz'f if sqegb`zz'f`xx'q~=.
	replace cf`zz'rmse=sqrt(r(mean)) in `xx'
	}			
			
}

*Produce relative RMSE

foreach zz in inf gdpg intr dint {
	
	gen swgbrel`zz'rmse=sw`zz'rmse/gb`zz'rmse in 1/8
	gen swbvarrel`zz'rmse=sw`zz'rmse/bvar`zz'rmse in 1/8
}




**************
*Make FIGURES*
**************


*Produce Relative RMSE graphs--Figure 1

graph twoway scatter swgbrelinfrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. Inflation Forecast RMSE) name(sginfrmse)
graph twoway scatter swgbrelgdpgrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. GDP Growth Forecast RMSE) name(sggdpgrmse)
graph twoway scatter swbvarrelinfrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. Inflation Forecast RMSE) name(sbinfrmse)
graph twoway scatter swbvarrelgdpgrmse quarters in 1/8, yline(1) ylabel(0.5(0.5)1.5) msymbol(O) mcolor(black ) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Rel. GDP Growth Forecast RMSE) name(sbgdpgrmse)

graph combine sginfrmse sggdpgrmse, cols(1) title("DSGE Model Relative to GB") name(sg) ycommon 
graph combine sbinfrmse sbgdpgrmse, cols(1) title("DSGE Model Relative to BVAR") name(sb) ycommon
graph combine sg sb, cols(2) title("Figure 1. Relative Forecast RMSEs") name(Figure1) saving("Figure1.gph", replace)
graph export Figure1.wmf, replace


*Produce scatter graphs--Figure 2

scatter inf swinff4q if gbinff4q~=., xsc(r(0 1)) ysc(r(0 1)) xlabel(0(0.2)1) ylabel(0(0.2)1) title("Figure 2a. Realized and 4Q Ahead DSGE Inflation Forecast") xtitle("DSGE Model Forecast of Inflation") ytitle(Realized Inflation) name(Figure2a) saving("Figure2a.gph", replace)
graph export Figure2a.wmf, replace
scatter gdpg swgdpgf4q if gbinff4q~=., xlabel(-01(0.5)2) ylabel(-1(0.5)2) title("Figure 2b. Realized and 4Q Ahead DSGE GDP Growth Forecast")  xtitle("DSGE Model Forecast of GDP Growth") ytitle(Realized GDP Growth) name(Figure2b) saving("Figure2b.gph", replace)
graph export Figure2b.wmf, replace
*graph combine sw4qinf sw4qgdp, cols(1) title("Figure 2. Realized and 4Q Ahead DSGE Model Forecasts") name(Figure2) saving("Figure2.gph", replace)
*graph export Figure2.wmf, replace


*Produce RMSE graphs--Figure 3

graph twoway scatter swinfrmse bvarinfrmse gbinfrmse rwinfrmse cfinfrmse quarters in 1/8, msymbol(X sh Dh Th Oh) mcolor(black black black black) ylabel(0.1(0.1)0.4) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(Inflation) legend(off) name(infrmse)
graph twoway scatter swgdpgrmse bvargdpgrmse gbgdpgrmse rwgdpgrmse cfgdpgrmse quarters in 1/8, msymbol(X sh Dh Th Oh) mcolor(black black black black) ylabel(0.4(0.2)1) xlabel(1(1)8) xtitle("Quarters Ahead") ytitle(GDP Growth) legend(off) name(gdpgrmse)
graph twoway scatter swdintrmse bvardintrmse gbdintrmse rwdintrmse cfdintrmse quarters in 1/8, ylabel(0(0.05)0.15) legend(label(1 "DSGE") label(2 "BVAR") label(3 "Greenbook") label(4 "Random Walk") label(5 "Constant") rows(1) ring(0) position(5)) msymbol(X sh Dh Th Oh) mcolor(black black black black) xlabel(1(1)8) xtitle("Quarters") ytitle(Interest Rate Change) name(dintrmse)

graph combine infrmse gdpgrmse dintrmse, cols(1) title("Figure 3. RMSEs of Alternative Forecasts") name(Figure3) saving("Figure3.gph", replace)
graph export Figure3.wmf, replace

*Produce inflation graph--Figure 4

graph twoway line inf numdate if ab=="A", ytitle("Inflation") xtitle("Date") title("Figure 4. A Short History of Inflation") xlabel( #14, angle(vertical)) xline(20 131, lwidth(0.5)) xline(128 187, lwidth(1)) name(tsinf) saving(Figure4.gph, replace)
graph export Figure4.wmf, replace


*************
*Make TABLES*
*************

*The code will crash here if you have not installed outreg. 

foreach yy in sw bvar gb {

	foreach zz in inf gdpg intr dint {
	
	newey `zz', lag(1)
	outreg using `yy'_`zz'_newey.out,  coefastr se replace
	reg `zz'
	outreg using `yy'_`zz'_reg.out,  coefastr se replace
	
	}
}


forvalues xx=1/6 {

	foreach yy in sw bvar gb {

		foreach zz in inf gdpg intr dint {
		
		label var `yy'`zz'f`xx'q "`yy'`zz'fcast"
		local lag=`xx'*2
		newey `zz' `yy'`zz'f`xx'q, lag(`lag')
		outreg using `yy'_`zz'_newey.out, coefastr se append
		reg `zz' `yy'`zz'f`xx'q, cluster(yrq)
		outreg using `yy'_`zz'_reg.out, coefastr se append

		}
	}
}

*The code will crash here if you have not installed dmariano. 

gen swcfinf=.
gen swcfgdpg=.
gen swcfdint=.

forvalues xx=1/8 {

	foreach yy in inf gdpg dint {
	
	dmariano `yy' sw`yy'f`xx'q cf`yy'f if gb`yy'f`xx'q~=., kernel(bartlett)
	replace swcf`yy'=r(p) in `xx'
	}
}



*Well done. 

log close
